<?php
/**
 * Main application class
 *
 * @package wp-swift-page
 * @author InstaProduct
 * @link http://insta-product.com/
 */

//
//Include all function files
require_once( IPOPTZ_LIB . '/config.php' );
require_once( IPOPTZ_LIB . '/setting.php' );
require_once( IPOPTZ_LIB . '/image.php' );
require_once( IPOPTZ_LIB . '/core.php' );
require_once( IPOPTZ_LIB . '/request.php' );
require_once( IPOPTZ_APP . '/controller.php' );
require_once( IPOPTZ_APP . '/viewer.php' );
require_once( IPOPTZ_LIB . '/metabox.php' );
require_once( IPOPTZ_LIB . '/googleLib.php' );
require_once( IPOPTZ_LIB . '/Services_WTF_Test.php' );

if ( ! class_exists( 'IPOptz_App' ))
{
	class IPOptz_App
	{
		//{{ run

		/**
		 * Run all functions
		 *
		 * @return void
		 */
		public static function run()
		{
			$hook = sanitize_title( IPOPTZ_TITLE );

			//
			//Rewrite

      //Install Plugin
      register_activation_hook( __FILE__, array( __CLASS__, 'install' ) );

      //App Init
      add_action( 'init', array( __CLASS__, 'appInit' ) );

      //Render page
      add_action( 'template_redirect', array( __CLASS__, 'renderRequest' ) );

			//
			//Admin init
			add_action( 'admin_init', array( __CLASS__, 'adminInit' ) );

			//
			//Register admin menu
			add_action( 'admin_menu', array( __CLASS__, 'addAdminMenu' ) );

			//
			//Count page speed
			add_action( 'wp_footer', array( __CLASS__, 'setPageStats' ) );

			//
			// Add meta box option
			//
			add_action( 'add_meta_boxes', array( __CLASS__, 'addMetaBox' ) );
			add_action( 'save_post', array( __CLASS__, 'savePost' ) );
			
			//
			//Add css
			add_action(
				'admin_print_styles-toplevel_page_' . IPOPTZ_PAGENAME,
				array( __CLASS__, 'adminStyle' ),
				1100
			);
			add_action(
				'admin_print_styles-'.$hook.'_page_' . IPOPTZ_PAGENAME . '-script',
				array( __CLASS__, 'adminStyle' ),
				1100
			);
			add_action(
				'admin_print_styles-'.$hook.'_page_' . IPOPTZ_PAGENAME . '-style',
				array( __CLASS__, 'adminStyle' ),
				1100
			);
			add_action(
				'admin_print_styles-'.$hook.'_page_' . IPOPTZ_PAGENAME . '-general',
				array( __CLASS__, 'adminStyle' ),
				1100
			);
			add_action(
				'admin_print_styles-'.$hook.'_page_' . IPOPTZ_PAGENAME . '-image',
				array( __CLASS__, 'adminStyle' ),
				1100
			);
			add_action(
				'admin_print_styles-'.$hook.'_page_' . IPOPTZ_PAGENAME . '-setting',
				array( __CLASS__, 'adminStyle' ),
				1100
			);
			add_action(
				'admin_print_styles',
				array( __CLASS__, 'adminGlobalStyle' ),
				1000
			);

			//
			//Add javascript
			add_action(
				'admin_print_scripts-toplevel_page_' . IPOPTZ_PAGENAME,
				array( __CLASS__, 'adminScript' ),
				1100
			);
			add_action(
				'admin_print_scripts-'.$hook.'_page_'.IPOPTZ_PAGENAME. '-script',
				array( __CLASS__, 'adminScript' ),
				1100
			);
			add_action(
				'admin_print_scripts-'.$hook.'_page_'.IPOPTZ_PAGENAME. '-style',
				array( __CLASS__, 'adminScript' ),
				1100
			);
			add_action(
				'admin_print_scripts-'.$hook.'_page_'.IPOPTZ_PAGENAME. '-general',
				array( __CLASS__, 'adminScript' ),
				1100
			);
			add_action(
				'admin_print_scripts-'.$hook.'_page_'.IPOPTZ_PAGENAME. '-image',
				array( __CLASS__, 'adminScript' ),
				1100
			);
			add_action(
				'admin_print_scripts-'.$hook.'_page_'.IPOPTZ_PAGENAME. '-setting',
				array( __CLASS__, 'adminScript' ),
				1100
			);
			add_action(
				'admin_print_scripts',
				array( __CLASS__, 'adminGlobalScript' ),
				1000
			);

			if ( ! is_admin() ) {
        add_action('wp_enqueue_scripts', array( __CLASS__, 'publicScript' ), 10 );
      }

			//
			//Run core function
			IPOptz_Core::init();
		}

		//}}
		//{{ adminInit

		/**
		 * Admin init
		 *
		 * @return void
		 */
		public static function adminInit()
		{
			/**
			 * Ajax handler
			 **/
			$controller = new IPOptz_Controller();
			$controller->ajax();

			$url = get_site_url();
			$url = add_query_arg( 'ipoptz_admin', 1, $url );
      $response = wp_remote_get( $url, array() );
		}

		//}}
		//{{ addAdminMenu

		/**
		 * Add admin menu
		 *
		 * @return void
		 */
		public static function addAdminMenu()
		{
			add_menu_page(
				IPOPTZ_TITLE,
				IPOPTZ_TITLE,
				'edit_posts',
				IPOPTZ_PAGENAME,
				array( __CLASS__, 'dashboard' ),
				'dashicons-chart-area'
			);
			add_submenu_page(
				IPOPTZ_PAGENAME,
				__( 'Dashboard', IPOPTZ_SLUG ),
				__( 'Dashboard', IPOPTZ_SLUG ),
				'edit_posts',
				IPOPTZ_PAGENAME,
				array( __CLASS__, 'dashboard' )
			);
			add_submenu_page(
				IPOPTZ_PAGENAME,
				__( 'General', IPOPTZ_SLUG ),
				__( 'General', IPOPTZ_SLUG ),
				'edit_posts',
				IPOPTZ_PAGENAME . '-general',
				array( __CLASS__, 'general' )
			);
			add_submenu_page(
				IPOPTZ_PAGENAME,
				__( 'Script', IPOPTZ_SLUG ),
				__( 'Script', IPOPTZ_SLUG ),
				'edit_posts',
				IPOPTZ_PAGENAME . '-script',
				array( __CLASS__, 'script' )
			);
			add_submenu_page(
				IPOPTZ_PAGENAME,
				__( 'Style', IPOPTZ_SLUG ),
				__( 'Style', IPOPTZ_SLUG ),
				'edit_posts',
				IPOPTZ_PAGENAME . '-style',
				array( __CLASS__, 'style' )
			);
			add_submenu_page(
				IPOPTZ_PAGENAME,
				__( 'Image', IPOPTZ_SLUG ),
				__( 'Image', IPOPTZ_SLUG ),
				'edit_posts',
				IPOPTZ_PAGENAME . '-image',
				array( __CLASS__, 'image' )
			);
			add_submenu_page(
				IPOPTZ_PAGENAME,
				__( 'Settings', IPOPTZ_SLUG ),
				__( 'Settings', IPOPTZ_SLUG ),
				'edit_posts',
				IPOPTZ_PAGENAME . '-setting',
				array( __CLASS__, 'setting' )
			);
		}

		//}}
		//{{ dashboard

		/**
		 * General optimizing management
		 *
		 * @return void
		 */
		public static function dashboard()
		{
			$viewer = new IPOptz_Viewer();
			$viewer->dashboard();
		}

		//}}
		//{{ general

		/**
		 * General optimizing management
		 *
		 * @return void
		 */
		public static function general()
		{
			$viewer = new IPOptz_Viewer();
			$viewer->general();
		}

		//}}
		//{{ script

		/**
		 * Script optimizing management
		 *
		 * @return void
		 */
		public static function script()
		{
			$viewer = new IPOptz_Viewer();
			$viewer->script();
		}

		//}}
		//{{ style

		/**
		 * Style optimizing management
		 *
		 * @return void
		 */
		public static function style()
		{
			$viewer = new IPOptz_Viewer();
			$viewer->style();
		}

		//}}
		//{{ image

		/**
		 * Image optimizing management
		 *
		 * @return void
		 */
		public static function image()
		{
			$viewer = new IPOptz_Viewer();
			$viewer->image();
		}

		//}}
		//{{ setting

		/**
		 * Setting page
		 *
		 * @return void
		 */
		public static function setting()
		{
			$viewer = new IPOptz_Viewer();
			$viewer->setting();
		}

    //}}
    //{{ install

    /**
     * Install plugin
     *
     * @return void
     */
    public static function install()
    {
      //Rewrite
      self::registerRewrite();
      flush_rewrite_rules();

      //Set speed
      $timerStop  = timer_stop( 0, 2 );
			$numQueries = get_num_queries();

			if ( get_option('ipoptz_page_load_time') == '' ) {
				update_option( 'ipoptz_page_load_time', $timerStop );
			}

			if ( get_option( 'ipoptz_page_num_queries') == '' ) {
				update_option( 'ipoptz_page_num_queries', $numQueries );
			}
    }

    //}}
    //{{ setPageStats

    /**
     * Set page stats
     *
     * @return void
     */
    public static function setPageStats()
    {
    	if ( isset( $_GET['ipoptz_admin'] ))
    	{
	      //Set speed
	      $timerStop  = timer_stop( 0, 2 );
				$numQueries = get_num_queries();

				update_option( 'ipoptz_page_load_time', $timerStop );
				update_option( 'ipoptz_page_num_queries', $numQueries );

      	wp_enqueue_scripts();

				//Save scripts list
				$scripts = IPOptz_Core::getRegisteredScripts();
				update_option( '__ipoptz_registered_scripts', $scripts );

				//Save styles list
				$styles = IPOptz_Core::getRegisteredStyles();
				update_option( '__ipoptz_registered_styles', $styles );
    	}
    }

    //}}
    //{{ appInit

    /**
     * App init
     *
     * @return void
     */
    public static function appInit()
    {
      //Rewrite
      self::rewriteFlush();
    }

    //}}
    //{{ registerRewrite

    /**
     * Init rewrite rule
     *
     * @return void
     */
    public static function registerRewrite()
    {
      add_rewrite_rule(
        'ipoptz-request/getstyle/?$',
        'index.php?ipoptz=1&ipoptz_action=getstyle',
        'top'
      );
      add_rewrite_rule(
        'ipoptz-request/getscript/?$',
        'index.php?ipoptz=1&ipoptz_action=getscript',
        'top'
      );
      add_rewrite_rule(
        'ipoptz-request/getstyle/?([0-9]{1,})/?$',
        'index.php?ipoptz=1&ipoptz_action=getstyle&ipoptz_post=$matches[1]',
        'top'
      );
      add_rewrite_rule(
        'ipoptz-request/getscript/?([0-9]{1,})/?$',
        'index.php?ipoptz=1&ipoptz_action=getscript&ipoptz_post=$matches[1]',
        'top'
      );
      add_rewrite_tag( '%ipoptz%', '([1])');
      add_rewrite_tag( '%ipoptz_action%', '([a-zA-Z0-9\_])');
      add_rewrite_tag( '%ipoptz_post%', '([0-9])');
    }

    //}}
    //{{ rewriteFlush

    /**
     * Flush rewrite rule
     *
     * @return void
     */
    public static function rewriteFlush()
    {
      self::registerRewrite();
    }

    //}}
    //{{ renderRequest

    /**
     * Render request
     *
     * @return void
     */
    public static function renderRequest()
    {
      global $wp_query;

    	if ( isset( $wp_query->query['ipoptz'] ) && 
    			 ! empty( $wp_query->query['ipoptz_action'] )
    	) {
    		$action  = apply_filters( 'pre_term_name', $wp_query->query['ipoptz_action'] );
    		$request = new IPOptz_Request( $action );
    		$request->render();
    	}
    }

		//}}
		//{{ addMetaBox

		/**
		 * Add administrator meta box
		 *
		 * @return void
		 */
		public static function addMetaBox()
		{
			add_meta_box(
				'ipeng_metabox',
				__( 'WP Optimizing', IPENG_SLUG ),
				array( __CLASS__, 'metaBoxContent' ),
				'page'
			);
		}

		//}}
		//{{ metaBoxContent

		/**
		 * Content for administrator meta box
		 *
		 * @return void
		 */
		public static function metaBoxContent( $post )
		{
			$viewer = new IPOptz_Metabox();
			$viewer->viewer( $post );
		}

		//}}
		//{{ savePost

		/**
		 * Save page/post meta box setting
		 *
		 * @param  int $postId
		 * @return void
		 */
		public static function savePost( $postId )
		{
			$saver = new IPOptz_Metabox();
			$saver->save( $postId );
		}

		//}}
		//{{ adminStyle

		/**
		 * Add Css file
		 *
		 * @return void
		 */
		public static function adminStyle()
		{
			wp_deregister_style( 'ipoptz-admin-style' );
			wp_register_style(
				'ipoptz-admin-style',
				IPOPTZ_BASE_URL . 'assets/css/admin.min.css',
				false,
				null
			);
			wp_enqueue_style('ipoptz-admin-style');
		}

		//}}
		//{{ adminGlobalStyle

		/**
		 * Add Css file to all pages
		 *
		 * @return void
		 */
		public static function adminGlobalStyle()
		{
			wp_deregister_style( 'ipoptz-admin-global-style' );
			wp_register_style(
				'ipoptz-admin-global-style',
				IPOPTZ_BASE_URL . 'assets/css/admin-global.min.css',
				false,
				null
			);
			wp_enqueue_style('ipoptz-admin-global-style');
		}

		//}}
		//{{ adminScript

		/**
		 * Add javascript file
		 *
		 * @return void
		 */
		public static function adminScript()
		{
		?>
			<script type="text/javascript">
				var ipoptz_img_url = '<?php echo IPOPTZ_BASE_URL . 'assets/img/'; ?>',
					ui_base_image	= '<?php echo IPOPTZ_BASE_URL . 'assets/img/'; ?>',
					ui_message		 =
					{
						delete_confirm_title: '<?php _e( 'Delete Confirmation', IPOPTZ_SLUG ); ?>',
						delete_confirm_info: '<?php _e( 'Are you sure doing this action?', IPOPTZ_SLUG ); ?>'
					}
			</script>

			<?php
			wp_deregister_script('jquery-ui');
			wp_register_script(
				'jquery-ui',
				"http" . ($_SERVER['SERVER_PORT'] == 443 ? "s" : "") . "://ajax.googleapis.com/ajax/libs/jqueryui/1.10.4/jquery-ui.min.js",
				false,
				null
			);
			wp_enqueue_script('jquery-ui');

			wp_deregister_script('ipoptz-admin-script');
			wp_enqueue_script(
				'ipoptz-admin-script',
				IPOPTZ_BASE_URL . 'assets/js/admin.js',
				array( 'jquery' ),
				1,
				true
			);
		}

		//}}
		//{{ adminGlobalScript

		/**
		 * Add javascript file to all pages
		 *
		 * @return void
		 */
		public static function adminGlobalScript()
		{
			wp_deregister_script('ipoptz-admin-global-script');
			wp_enqueue_script(
				'ipoptz-admin-global-script',
				IPOPTZ_BASE_URL . 'assets/js/admin-global.min.js',
				array( 'jquery' ),
				1,
				true
			);
		}

    //}}
    //{{ publicScript

    /**
     * Print public script
     *
     * @return void
     */
    public static function publicScript()
    {
      wp_enqueue_script('jquery');
      wp_deregister_script('jquery-lazyload');
      wp_enqueue_script(
        'jquery-lazyload',
        IPOPTZ_BASE_URL . 'assets/js/third_party/jquery.lazyload.js',
				array( 'jquery' ),
        1,
        true
      );

      wp_deregister_script('insta-ipoptz-public-script');
      wp_enqueue_script(
        'insta-ipoptz-public-script',
        IPOPTZ_BASE_URL . 'assets/js/public.min.js',
        array( 'jquery' ),
        1,
        true
      );
    }
		//}}
	}
}
