<?php
if ( ! class_exists( 'IPOptz_GoogleLib' )) 
{
	/**
	 * Change wordpress assets to google libraris
	 *
	 * @package wp-swift-page
	 * @author InstaProduct
	 * @link http://insta-product.com/
	 */

	class IPOptz_GoogleLib
	{
		/**
		 * Google libraries list
		 *
		 * @var array
		 */
		public static $libraries = array
		(
			/* jQuery */
			'jquery' => array( 'jquery', 'jquery.min', '' ),

			/* jQuery UI */
			'jquery-ui-core' => array( 'jqueryui', 'jquery-ui.min', '' ),
			'jquery-ui-accordion' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-autocomplete' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-button' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-datepicker' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-dialog' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-draggable' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-droppable' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-menu' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-mouse' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-position' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-progressbar' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-resizable' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-selectable' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-slider' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-sortable' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-tabs' => array( '', '', 'jquery-ui-core' ),
			'jquery-ui-widget' => array( '', '', 'jquery-ui-core' ),

			/* jQuery Effects */
			'jquery-effects-core' => array( '', '', 'jquery-ui-core' ),
			'jquery-effects-blind' => array( '', '', 'jquery-ui-core' ),
			'jquery-effects-bounce' => array( '', '', 'jquery-ui-core' ),
			'jquery-effects-clip' => array( '', '', 'jquery-ui-core' ),
			'jquery-effects-drop' => array( '', '', 'jquery-ui-core' ),
			'jquery-effects-explode' => array( '', '', 'jquery-ui-core' ),
			'jquery-effects-fade' => array( '', '', 'jquery-ui-core' ),
			'jquery-effects-fold' => array( '', '', 'jquery-ui-core' ),
			'jquery-effects-highlight' => array( '', '', 'jquery-ui-core' ),
			'jquery-effects-pulsate' => array( '', '', 'jquery-ui-core' ),
			'jquery-effects-scale' => array( '', '', 'jquery-ui-core' ),
			'jquery-effects-shake' => array( '', '', 'jquery-ui-core' ),
			'jquery-effects-slide' => array( '', '', 'jquery-ui-core' ),
			'jquery-effects-transfer' => array( '', '', 'jquery-ui-core' ),

			/* prototype */
			'prototype' => array( 'prototype', 'prototype', '' ),

			/* scriptaculous */
			'scriptaculous-root' => array( 'scriptaculous', 'scriptaculous', '' ),
			'scriptaculous-builder' => array( '', '', 'scriptaculous-root' ),
			'scriptaculous-effects' => array( '', '', 'scriptaculous-root' ),
			'scriptaculous-dragdrop' => array( '', '', 'scriptaculous-root' ),
			'scriptaculous-controls' => array( '', '', 'scriptaculous-root' ),
			'scriptaculous-slider' => array( '', '', 'scriptaculous-root' ),
			'scriptaculous-sound' => array( '', '', 'scriptaculous-root' ),

			/* moo tools */
			'mootools' => array( 'mootools', 'mootools-yui-compressed', '' ),

			/* Dojo */
			'dojo' => array( 'dojo', 'dojo.xd', '' ),

			/* swfobject */
			'swfobject' => array( 'swfobject', 'swfobject', '' ),

			/* YUI */
			'yui' => array( 'yui', 'build/yuiloader/yuiloader-min', '' ),

			/* Ext Core */
			'ext-core' => array( 'ext-core', 'ext-core', '' )
		);

		/**
		 * No conflict script printing flag
		 *
		 * @var boolean
		 */
		public static $noconflictPrint = false;

		/**
		 * Take jquery tag
		 *
		 * @var string
		 */
		public static $jqueryTag = 'jquery';

		/**
		 * Class init
		 *
		 * @return void
		 */
		public static function init()
		{
			add_action( 'wp_default_scripts', array( __CLASS__, 'replaceDefaultScripts' ), 1000 );
			add_filter( 'script_loader_src', array( __CLASS__, 'replaceQueryVer' ), 1000 );
			add_filter( 'init', array( __CLASS__, 'initFilter' ) );

			//Run again
			global $wp_scripts;
			if ( is_a( $wp_scripts, 'WP_Scripts' ) ) 
			{
				self::replaceDefaultScripts( $wp_scripts );
			}
		}

		/**
		 * Replacing default scripts
		 *
		 * @param array $WP_Scripts
		 * @return void
		 */
		public static function replaceDefaultScripts( &$WP_Scripts )
		{
			$glscripts = self::getGLScripts( $WP_Scripts );
			foreach ( $glscripts as $script ) 
			{
				$oldData = self::getDependencyData( $WP_Scripts, $script->handle );
				$WP_Scripts->remove( $script->handle );
				// re-register script
				$WP_Scripts->add( $script->handle, $script->src, $script->deps, $script->ver );
				if ( $oldData ) 
				{
					foreach ( $oldData as $name => $value ) {
						$WP_Scripts->add_data( $script->handle, $name, $value );
					}
				}
			}
		}

		/**
		 * Get dependency data
		 *
		 * @param array  $WP_Scripts
		 * @param string $handle
		 * @return void
		 */
		public static function getDependencyData( $WP_Scripts, $handle )
		{
			if ( ! method_exists( $WP_Scripts, 'add_data' ) ) {
				return false;
			}
			if ( ! isset( $WP_Scripts->registered[$handle] ) ) {
				return false;
			}

			return $WP_Scripts->registered[$handle]->extra;
		}

		/**
		 * Get google library scripts
		 *
		 * @param array  $WP_Scripts
		 * @return void
		 */
		public static function getGLScripts( $WP_Scripts )
		{
			$glscripts = self::buildGLScripts( $WP_Scripts );
			return $glscripts;
		}

		/**
		 * Build google library scripts
		 *
		 * @param array  $WP_Scripts
		 * @return void
		 */
		public static function buildGLScripts( $WP_Scripts )
		{
			$glscripts  = array();
			$comscripts = array();
			$libraries  = self::$libraries;

			self::fixJqueryCore( $WP_Scripts );

			foreach ( $libraries as $name => $libdata ) 
			{
				if ( $script = $WP_Scripts->query( $name ) ) 
				{
					$libname  = $libdata[0];
					$libjs    = $libdata[1];
					$combined = $libdata[2];
					$version  = $script->ver;

					if ( strpos( $version, '-' ) !== false ) {
						continue;
					}

					// Fix for scriptaculous 1.8.0
					if ( $name == 'scriptaculous-root' && $version == '1.8.0' ) {
						$version = '1.8';
					}

					if ( $combined !== '' ) 
					{
						if ( ! in_array( $combined, $comscripts ) ) {
							continue;
						}
						if ( ! in_array( $combined, $script->deps ) ) {
							$script->deps[] = $combined;
						}
					}

					$script->src = self::buildGLScriptUrl(
						$name, $libname, $version, $libjs, $script->src
					);

					$glscripts[]  = $script;
					$comscripts[] = $name;
				}
			}

			return $glscripts;
		}

		/**
		 * Build google library script URL
		 *
		 * @param string $handle
		 * @param string $libname
		 * @param string $version
		 * @param string $jsfile
		 * @param string $src
		 * @return void
		 */
		public static function buildGLScriptUrl( $handle, $libname, $version, $jsfile, $src )
		{
			if ( $libname != '' ) 
			{
				// build google library URL
				$url  = "//ajax.googleapis.com/ajax/libs/$libname/$version/$jsfile.js";
				$head = wp_remote_head( "http:$url" );
				if ( wp_remote_retrieve_response_code( $head ) !== 200 ) {
					return $src;
				}
				return $url;
			} 
			else {
				return '';
			}
		}

		/**
		 * Fix JQuery core
		 *
		 * @param array  $WP_Scripts
		 * @return void
		 */
		public static function fixJqueryCore( $WP_Scripts )
		{
			$libs = self::$libraries;
			if ( $WP_Scripts->query( 'jquery-core' ) && array_key_exists( 'jquery', $libs ) ) 
			{
				self::$libraries['jquery-core'] = $libs['jquery'];
				unset( self::$libraries['jquery'] );
				self::$jqueryTag = 'jquery-core';
			}
		}

		/**
		 * Replacing default scripts
		 *
		 * @param  string $src
		 * @return void
		 */
		public static function replaceQueryVer( $src )
		{
			if ( self::$noconflictPrint ) {
				self::$noconflictPrint = false;
				echo self::noconflictScript();
			}

			$pattern = '/ajax\.googleapis\.com\//';
			if ( preg_match( $pattern, $src ) ) 
			{
				$src = remove_query_arg( 'ver', $src );
				$tag = self::$jqueryTag;
				if ( strpos( $src, self::$libraries[$tag][1] . '.js' ) ) {
					self::$noconflictPrint = true;
				}
			}
			return $src;
		}

		/**
		 * Replacing default scripts
		 *
		 * @return string
		 */
		public static function noconflictScript()
		{
			$code = "<script type='text/javascript'>try{jQuery.noConflict();}catch(e){};</script>\n";

			return $code;
		}

		/**
		 * Init filter
		 *
		 * @return string
		 */
		public static function initFilter()
		{
			global $concatenate_scripts;
			$concatenate_scripts = false;
		}
	}
}